#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Chop Zone Sideways Market Indicator
    /// </summary>
    [Description("Chop Zone Sideways Market Indicator")]
    public class CZI : Indicator
    {
        #region Variables
        public int		chopCrosses		= 4;
		private int 	chopXings		= 0;
		private int 	periods			= 14;
		private int		resetbar;
		private int		sidewaysbar;
		private int		sidewaysReset	= 200;
		private bool	sideways		= false;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(Color.FromKnownColor(KnownColor.Orange), PlotStyle.Line, "Plot0"));
            CalculateOnBarClose	= true;
            Overlay				= false;
            PriceTypeSupported	= false;
			
			WoodiesCCI myCCI = WoodiesCCI(2, 5, 14, 34, 25, 6, 30, 100, 2);
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (CurrentBar < Periods) { return; }
			
			WoodiesCCI myCCI = WoodiesCCI(2, 5, 14, 34, 25, 6, 30, 100, 2);
			int BarsAgo_n100Xdown 	= MRO(delegate{return (myCCI[0] < -100 && (myCCI[Math.Min(CurrentBar, 1)]==-100 ? myCCI[Math.Min(CurrentBar, 2)]>-100 : myCCI[Math.Min(CurrentBar, 1)]>-100));},1,CurrentBar) +1;// -100 line cross up
			int BarsAgo_100Xup 		= MRO(delegate{return (myCCI[0] > 100 && (myCCI[Math.Min(CurrentBar, 1)]==0 ? myCCI[Math.Min(CurrentBar, 2)]<100 : myCCI[Math.Min(CurrentBar, 1)]<100));},1,CurrentBar) +1;// 100 line cross up
			int BarsAgoSidewaysresetdown = MRO(delegate{return (myCCI[0]<SidewaysReset && (myCCI[Math.Min(CurrentBar, 1)]==SidewaysReset ? myCCI[Math.Min(CurrentBar, 2)]>SidewaysReset : myCCI[Math.Min(CurrentBar, 1)]>SidewaysReset));},1,CurrentBar) +1;// Extreme cross down
			int BarsAgoSidewaysresetup 	= MRO(delegate{return (myCCI[0]>-SidewaysReset && (myCCI[Math.Min(CurrentBar, 1)]==-SidewaysReset ? myCCI[Math.Min(CurrentBar, 2)]<-SidewaysReset : myCCI[Math.Min(CurrentBar, 1)]<-SidewaysReset));},1,CurrentBar) +1;// Extreme cross down
			
			
			if(FirstTickOfBar)
			{
				if(chopXings<ChopCrosses)
				{
					chopXings = CountIf(
						delegate{
							return((myCCI[0]>0 && myCCI[1]<0) || (myCCI[0]<0 && myCCI[1]>0));
						},Math.Min(BarsAgo_n100Xdown, BarsAgo_100Xup));
					sidewaysbar = CurrentBar;//Set bar # of sideways market start
				}
				
				if((BarsAgoSidewaysresetdown<=(CurrentBar-sidewaysbar)&& BarsAgoSidewaysresetdown!=0) || (BarsAgoSidewaysresetup<=(CurrentBar-sidewaysbar) && BarsAgoSidewaysresetup!=0))
				{
					chopXings = 0;
					resetbar = CurrentBar;//Set bar # of sideways market end
				}
				
				if (chopXings == ChopCrosses) {
					sideways = true;
				}
				else {
					sideways = false;
				}
			}
			
			if(chopXings == ChopCrosses) 
				DrawTextFixed("chop","SIDEWAYS MARKET",TextPosition.TopRight);
			else
				RemoveDrawObject("chop");
        }

        #region Properties
		[Description("CCI Periods")]
		[Category("Parameters")]
		[Gui.Design.DisplayName("CCI Periods")]
		public int Periods
		{
			get { return periods; }
			set { periods = Math.Max(2,value); }
		}
				
        [Description("Numbers of 0 line crosses needed to declare a sideways market")]
		[Category("Sound and Display")]
		[Gui.Design.DisplayName("Sideways mkt crosses")]
		public int ChopCrosses
		{
			get { return chopCrosses; }
			set { chopCrosses = Math.Max(1, value); }
		}
		
		[Description("CCI Level to reach to reset sideways market")]
		[Category("Sound and Display")]
		[Gui.Design.DisplayName("Sideways mkt reset level")]
		public int SidewaysReset
		{
			get { return sidewaysReset; }
			set { sidewaysReset = Math.Max(1, value); }
		}
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public bool Sideways
        {
            get { return sideways; }
        }
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private CZI[] cacheCZI = null;

        private static CZI checkCZI = new CZI();

        /// <summary>
        /// Chop Zone Sideways Market Indicator
        /// </summary>
        /// <returns></returns>
        public CZI CZI(int periods)
        {
            return CZI(Input, periods);
        }

        /// <summary>
        /// Chop Zone Sideways Market Indicator
        /// </summary>
        /// <returns></returns>
        public CZI CZI(Data.IDataSeries input, int periods)
        {
            checkCZI.Periods = periods;
            periods = checkCZI.Periods;

            if (cacheCZI != null)
                for (int idx = 0; idx < cacheCZI.Length; idx++)
                    if (cacheCZI[idx].Periods == periods && cacheCZI[idx].EqualsInput(input))
                        return cacheCZI[idx];

            CZI indicator = new CZI();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Periods = periods;
            indicator.SetUp();

            CZI[] tmp = new CZI[cacheCZI == null ? 1 : cacheCZI.Length + 1];
            if (cacheCZI != null)
                cacheCZI.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheCZI = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Chop Zone Sideways Market Indicator
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.CZI CZI(int periods)
        {
            return _indicator.CZI(Input, periods);
        }

        /// <summary>
        /// Chop Zone Sideways Market Indicator
        /// </summary>
        /// <returns></returns>
        public Indicator.CZI CZI(Data.IDataSeries input, int periods)
        {
            return _indicator.CZI(input, periods);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Chop Zone Sideways Market Indicator
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.CZI CZI(int periods)
        {
            return _indicator.CZI(Input, periods);
        }

        /// <summary>
        /// Chop Zone Sideways Market Indicator
        /// </summary>
        /// <returns></returns>
        public Indicator.CZI CZI(Data.IDataSeries input, int periods)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.CZI(input, periods);
        }

    }
}
#endregion
