#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Rahul Mohindra Indicator
    /// </summary>
    [Description("Rahul Mohindar Indicator v2.1.1. Ported to NinjaTrader by Randy Terbush. randy@terbush.org")]
	[Gui.Design.DisplayName("RMO Indicator v2.1.1")]	
    public class RMO : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int len1 = 2; // Default setting for Len1
            private int len2 = 10; // Default setting for Len2
            private int len3 = 30; // Default setting for Len3
            private int len4 = 81; // Default setting for Len4
			private int 	rmoShortEntryLevel = -7;
			private int 	rmoLongEntryLevel = 7;
		
			// User defined variables (add any user defined variables below)
			private DataSeries	vST1;
			private DataSeries	vST2;
			private DataSeries	vST3;
			private DataSeries	vRMO;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(new Pen(Color.Blue,2), PlotStyle.Bar, "BullBar"));
			Add(new Plot(new Pen(Color.Red,2), PlotStyle.Bar, "BearBar"));
			Add(new Plot(new Pen(Color.Gray,2), PlotStyle.Bar, "GrayBar"));
			Add(new Plot(new Pen(Color.Blue,2), PlotStyle.Line, "BullLine"));
			Add(new Plot(new Pen(Color.Red,2), PlotStyle.Line, "BearLine"));
			Add(new Line(Color.FromKnownColor(KnownColor.Gold), 0, "ZeroLine"));
			Lines[0].Pen.DashStyle = DashStyle.Dot;
			Lines[0].Pen.Width = 2;
			
			vST1 = new DataSeries(this);
			vST2 = new DataSeries(this);
			vST3 = new DataSeries(this);
			vRMO = new DataSeries(this);

            CalculateOnBarClose	= false;
            Overlay				= false;
            PriceTypeSupported	= false;
			DrawOnPricePanel	= false;
			PaintPriceMarkers	= false;
			AutoScale 			= false;
        }
		
		
        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (CurrentBar < len4) {
				return;
			}

			DrawHorizontalLine("RMOLongEntry", false, RMOLongEntryLevel, Color.Yellow, DashStyle.Dot, 1);
			DrawHorizontalLine("RMOShortEntry", false, RMOShortEntryLevel, Color.Yellow, DashStyle.Dot, 1);
			
			VarST1.Set(100 *
				(Close[0] - ((
				SMA(Close,Len1)[0] +
				SMA(SMA(Close,Len1),Len1)[0] +
				SMA(SMA(SMA(Close,Len1),Len1),Len1)[0] +
				SMA(SMA(SMA(SMA(Close,Len1),Len1),Len1),Len1)[0] +
				SMA(SMA(SMA(SMA(SMA(Close,Len1),Len1),Len1),Len1),Len1)[0] +
				SMA(SMA(SMA(SMA(SMA(SMA(Close,Len1),Len1),Len1),Len1),Len1),Len1)[0] +
				SMA(SMA(SMA(SMA(SMA(SMA(SMA(Close,Len1),Len1),Len1),Len1),Len1),Len1),Len1)[0] +
				SMA(SMA(SMA(SMA(SMA(SMA(SMA(SMA(Close,Len1),Len1),Len1),Len1),Len1),Len1),Len1),Len1)[0] +
				SMA(SMA(SMA(SMA(SMA(SMA(SMA(SMA(SMA(Close,Len1),Len1),Len1),Len1),Len1),Len1),Len1),Len1),Len1)[0] +
				SMA(SMA(SMA(SMA(SMA(SMA(SMA(SMA(SMA(SMA(Close,Len1),Len1),Len1),Len1),Len1),Len1),Len1),Len1),Len1),Len1)[0]) / 10))
				/ (High[HighestBar(Close, Len2 - 1)] - Low[LowestBar(Close, Len2 - 1)]));
	

			double EMA1 = EMA(vST1, Len3)[0];
			double EMA2 = EMA(EMA(vST1, Len3), Len3)[0];
			VarST2.Set(EMA1 + (EMA1 - EMA2));
			
			double EMA3 = EMA(vST2, Len3)[0];
			double EMA4 = EMA(EMA(vST2, Len3), Len3)[0];
			VarST3.Set(EMA3 + (EMA3 - EMA4));
			
			double EMA5 = EMA(vST1, Len4)[0];
			double EMA6 = EMA(EMA(vST1, Len4), Len4)[0];
			VarRMO.Set(EMA5 + (EMA5 - EMA6));
	
			if ((VarRMO[0] > 0) && (VarST2[0] > 0) && (VarST3[0] > 0)) {
				BullLine.Set(VarRMO[0]);
				BullBar.Set(VarRMO[0]);
			}
			else if ((VarRMO[0] < 0) && (VarST2[0] < 0) && (VarST3[0] < 0)) {
				BearLine.Set(VarRMO[0]);
				BearBar.Set(VarRMO[0]);
			}
			else if (VarRMO[0] > 0) {
				BullLine.Set(VarRMO[0]);
				GrayBar.Set(VarRMO[0]);
			}
			else {
				BearLine.Set(VarRMO[0]);
				GrayBar.Set(VarRMO[0]);
			}
		}

        #region Properties
        [Description("")]
        [Category("Parameters")]
        public int Len1
        {
            get { return len1; }
            set { len1 = Math.Max(1, value); }
        }

        [Description("")]
        [Category("Parameters")]
        public int Len2
        {
            get { return len2; }
            set { len2 = Math.Max(1, value); }
        }

        [Description("")]
        [Category("Parameters")]
        public int Len3
        {
            get { return len3; }
            set { len3 = Math.Max(1, value); }
        }

        [Description("")]
        [Category("Parameters")]
        public int Len4
        {
            get { return len4; }
            set { len4 = Math.Max(1, value); }
        }
		
		[Description("RMO Long Entry Level")]
        [Category("Levels")]
        public int RMOLongEntryLevel
        {
            get { return rmoLongEntryLevel; }
            set { rmoLongEntryLevel = value; }
        }
		
		[Description("RMO Short Entry Level")]
        [Category("Levels")]
        public int RMOShortEntryLevel
        {
            get { return rmoShortEntryLevel; }
            set { rmoShortEntryLevel = value; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        private DataSeries BullBar
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        private DataSeries BearBar
        {
            get { return Values[1]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        private DataSeries GrayBar
        {
            get { return Values[2]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        private DataSeries BullLine
        {
            get { return Values[3]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        private DataSeries BearLine
        {
            get { return Values[4]; }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
      	[XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
		public DataSeries	VarST1
		{
            get { return vST1; }
		}
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
      	[XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
		public DataSeries	VarST2
		{
            get { return vST2; }
      	}
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
      	[XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
		public DataSeries	VarST3
		{
            get { return vST3; }
      	}
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
      	[XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
		public DataSeries	VarRMO
		{
            get { return vRMO; }
      	}
		
		
		#endregion
		
		#region Miscellaneous
		public override void GetMinMaxValues(ChartControl chartControl, ref double min, ref double max)
		{
			min = -10; // As far south as it can go.
			max = +10; // Set a top limit too.
		} 
		#endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private RMO[] cacheRMO = null;

        private static RMO checkRMO = new RMO();

        /// <summary>
        /// Rahul Mohindar Indicator v2.1.1. Ported to NinjaTrader by Randy Terbush. randy@terbush.org
        /// </summary>
        /// <returns></returns>
        public RMO RMO(int len1, int len2, int len3, int len4)
        {
            return RMO(Input, len1, len2, len3, len4);
        }

        /// <summary>
        /// Rahul Mohindar Indicator v2.1.1. Ported to NinjaTrader by Randy Terbush. randy@terbush.org
        /// </summary>
        /// <returns></returns>
        public RMO RMO(Data.IDataSeries input, int len1, int len2, int len3, int len4)
        {
            checkRMO.Len1 = len1;
            len1 = checkRMO.Len1;
            checkRMO.Len2 = len2;
            len2 = checkRMO.Len2;
            checkRMO.Len3 = len3;
            len3 = checkRMO.Len3;
            checkRMO.Len4 = len4;
            len4 = checkRMO.Len4;

            if (cacheRMO != null)
                for (int idx = 0; idx < cacheRMO.Length; idx++)
                    if (cacheRMO[idx].Len1 == len1 && cacheRMO[idx].Len2 == len2 && cacheRMO[idx].Len3 == len3 && cacheRMO[idx].Len4 == len4 && cacheRMO[idx].EqualsInput(input))
                        return cacheRMO[idx];

            RMO indicator = new RMO();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Len1 = len1;
            indicator.Len2 = len2;
            indicator.Len3 = len3;
            indicator.Len4 = len4;
            indicator.SetUp();

            RMO[] tmp = new RMO[cacheRMO == null ? 1 : cacheRMO.Length + 1];
            if (cacheRMO != null)
                cacheRMO.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheRMO = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Rahul Mohindar Indicator v2.1.1. Ported to NinjaTrader by Randy Terbush. randy@terbush.org
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.RMO RMO(int len1, int len2, int len3, int len4)
        {
            return _indicator.RMO(Input, len1, len2, len3, len4);
        }

        /// <summary>
        /// Rahul Mohindar Indicator v2.1.1. Ported to NinjaTrader by Randy Terbush. randy@terbush.org
        /// </summary>
        /// <returns></returns>
        public Indicator.RMO RMO(Data.IDataSeries input, int len1, int len2, int len3, int len4)
        {
            return _indicator.RMO(input, len1, len2, len3, len4);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Rahul Mohindar Indicator v2.1.1. Ported to NinjaTrader by Randy Terbush. randy@terbush.org
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.RMO RMO(int len1, int len2, int len3, int len4)
        {
            return _indicator.RMO(Input, len1, len2, len3, len4);
        }

        /// <summary>
        /// Rahul Mohindar Indicator v2.1.1. Ported to NinjaTrader by Randy Terbush. randy@terbush.org
        /// </summary>
        /// <returns></returns>
        public Indicator.RMO RMO(Data.IDataSeries input, int len1, int len2, int len3, int len4)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.RMO(input, len1, len2, len3, len4);
        }

    }
}
#endregion
