#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Rahul Mohindra Indicator
    /// </summary>
    [Description("Rahul Mohindar Swingtrade Indicator v2.0.2. Ported to NinjaTrader by Randy Terbush. randy@terbush.org")]
	[Gui.Design.DisplayName("RMO SwingTrader Indicator v2.0.2")]
    public class RMOSwing : Indicator
    {
        #region Variables
        // Wizard generated variables
            private int rsLen1 = 2; // Default setting for Len1
            private int rsLen2 = 10; // Default setting for Len2
            private int rsLen3 = 30; // Default setting for Len3
            private int rsLen4 = 81; // Default setting for Len4
		
//			private DataSeries	vST1;
//			private DataSeries	vST2;
//			private DataSeries	vST3;

        // User defined variables (add any user defined variables below)
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
            Add(new Plot(new Pen(Color.RoyalBlue,1), PlotStyle.Line, "ST2"));
			Add(new Plot(new Pen(Color.Red,1), PlotStyle.Line, "ST3"));
			
			Add(new Line(Color.Gold, 0, "ZeroLine"));
			Lines[0].Pen.DashStyle = DashStyle.Dot;
			Lines[0].Pen.Width = 1;
			
            CalculateOnBarClose	= false;
            Overlay				= false;
            PriceTypeSupported	= false;
			DrawOnPricePanel	= false;
			PaintPriceMarkers	= false;
			AutoScale 			= false;
        }

       
		/// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			if (CurrentBar < rsLen4) {
				return;
			}
			
			ST2.Set(ZeroLagTEMA(RMO(rsLen1,rsLen2,rsLen3,rsLen4).VarST2, 28)[0]);
			ST3.Set(RMO(rsLen1,rsLen2,rsLen3,rsLen4).VarST3[0]);
        }

        #region Properties
		[Description("")]
        [Category("Parameters")]
        public int RSLen1
        {
            get { return rsLen1; }
            set { rsLen1 = Math.Max(1, value); }
        }

        [Description("")]
        [Category("Parameters")]
        public int RSLen2
        {
            get { return rsLen2; }
            set { rsLen2 = Math.Max(1, value); }
        }

        [Description("")]
        [Category("Parameters")]
        public int RSLen3
        {
            get { return rsLen3; }
            set { rsLen3 = Math.Max(1, value); }
        }

        [Description("")]
        [Category("Parameters")]
        public int RSLen4
        {
            get { return rsLen4; }
            set { rsLen4 = Math.Max(1, value); }
        }
		
		[Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries ST2
        {
            get { return Values[0]; }
        }

        [Browsable(false)]	// this line prevents the data series from being displayed in the indicator properties dialog, do not remove
        [XmlIgnore()]		// this line ensures that the indicator can be saved/recovered as part of a chart template, do not remove
        public DataSeries ST3
        {
            get { return Values[1]; }
        }
		#endregion
		
		#region Miscellaneous
		public override void GetMinMaxValues(ChartControl chartControl, ref double min, ref double max)
		{
			min = -20; // As far south as it can go.
			max = +20; // Set a top limit too.
		} 
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private RMOSwing[] cacheRMOSwing = null;

        private static RMOSwing checkRMOSwing = new RMOSwing();

        /// <summary>
        /// Rahul Mohindar Swingtrade Indicator v2.0.2. Ported to NinjaTrader by Randy Terbush. randy@terbush.org
        /// </summary>
        /// <returns></returns>
        public RMOSwing RMOSwing(int rSLen1, int rSLen2, int rSLen3, int rSLen4)
        {
            return RMOSwing(Input, rSLen1, rSLen2, rSLen3, rSLen4);
        }

        /// <summary>
        /// Rahul Mohindar Swingtrade Indicator v2.0.2. Ported to NinjaTrader by Randy Terbush. randy@terbush.org
        /// </summary>
        /// <returns></returns>
        public RMOSwing RMOSwing(Data.IDataSeries input, int rSLen1, int rSLen2, int rSLen3, int rSLen4)
        {
            checkRMOSwing.RSLen1 = rSLen1;
            rSLen1 = checkRMOSwing.RSLen1;
            checkRMOSwing.RSLen2 = rSLen2;
            rSLen2 = checkRMOSwing.RSLen2;
            checkRMOSwing.RSLen3 = rSLen3;
            rSLen3 = checkRMOSwing.RSLen3;
            checkRMOSwing.RSLen4 = rSLen4;
            rSLen4 = checkRMOSwing.RSLen4;

            if (cacheRMOSwing != null)
                for (int idx = 0; idx < cacheRMOSwing.Length; idx++)
                    if (cacheRMOSwing[idx].RSLen1 == rSLen1 && cacheRMOSwing[idx].RSLen2 == rSLen2 && cacheRMOSwing[idx].RSLen3 == rSLen3 && cacheRMOSwing[idx].RSLen4 == rSLen4 && cacheRMOSwing[idx].EqualsInput(input))
                        return cacheRMOSwing[idx];

            RMOSwing indicator = new RMOSwing();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.RSLen1 = rSLen1;
            indicator.RSLen2 = rSLen2;
            indicator.RSLen3 = rSLen3;
            indicator.RSLen4 = rSLen4;
            indicator.SetUp();

            RMOSwing[] tmp = new RMOSwing[cacheRMOSwing == null ? 1 : cacheRMOSwing.Length + 1];
            if (cacheRMOSwing != null)
                cacheRMOSwing.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheRMOSwing = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Rahul Mohindar Swingtrade Indicator v2.0.2. Ported to NinjaTrader by Randy Terbush. randy@terbush.org
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.RMOSwing RMOSwing(int rSLen1, int rSLen2, int rSLen3, int rSLen4)
        {
            return _indicator.RMOSwing(Input, rSLen1, rSLen2, rSLen3, rSLen4);
        }

        /// <summary>
        /// Rahul Mohindar Swingtrade Indicator v2.0.2. Ported to NinjaTrader by Randy Terbush. randy@terbush.org
        /// </summary>
        /// <returns></returns>
        public Indicator.RMOSwing RMOSwing(Data.IDataSeries input, int rSLen1, int rSLen2, int rSLen3, int rSLen4)
        {
            return _indicator.RMOSwing(input, rSLen1, rSLen2, rSLen3, rSLen4);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Rahul Mohindar Swingtrade Indicator v2.0.2. Ported to NinjaTrader by Randy Terbush. randy@terbush.org
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.RMOSwing RMOSwing(int rSLen1, int rSLen2, int rSLen3, int rSLen4)
        {
            return _indicator.RMOSwing(Input, rSLen1, rSLen2, rSLen3, rSLen4);
        }

        /// <summary>
        /// Rahul Mohindar Swingtrade Indicator v2.0.2. Ported to NinjaTrader by Randy Terbush. randy@terbush.org
        /// </summary>
        /// <returns></returns>
        public Indicator.RMOSwing RMOSwing(Data.IDataSeries input, int rSLen1, int rSLen2, int rSLen3, int rSLen4)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.RMOSwing(input, rSLen1, rSLen2, rSLen3, rSLen4);
        }

    }
}
#endregion
