#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
    /// Rahul Mohindra Indicator
    /// </summary>
    [Description("RMO System Indicator v3.0.0 - written by Randy Terbush, randy@terbush.org")]
	[Gui.Design.DisplayName("RMO System Indicator v3.0.0")]	
    public class RMOSystem : Indicator
    {
        #region Variables
        // Wizard generated variables
			private int len1 = 2; // Default setting for Len1
            private int len2 = 10; // Default setting for Len2
            private int len3 = 30; // Default setting for Len3
            private int len4 = 81; // Default setting for Len4
			private int 	rmoShortEntryLevel = -7;
			private int 	rmoLongEntryLevel = 7;
			private int 	bpbShortExitLevel = 30;
			private int 	bpbLongExitLevel = 80;
			private	int		alertInterval 	= 60;
		
			private bool	soundAlert		= true;
			private bool	colorBars		= true;
			private bool	shortEntry		= false;
			private bool	longEntry		= false;
			private bool	flatPosition	= true;
			
			private string	entryAlertFile	= "Alert4.wav";
			private string	exitAlertFile	= "Alert1.wav";
		
			private RMO myRMO;
        #endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {			
            CalculateOnBarClose	= false;
            Overlay				= true;
            PriceTypeSupported	= false;
			DrawOnPricePanel	= true;
			PaintPriceMarkers	= false;
			AutoScale 			= false;
        }
		
		
        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {
			RMO myRMO = RMO(Len1,Len2,Len3,Len4);
			CZI myCZI = CZI(Len3);
			
			int downArrowA 	= 0;
			int downArrowB 	= 0;
			int downArrowC	= 0;
			int upArrowA 	= 0;
			int upArrowB	= 0;
			int upArrowC	= 0;
			
			if (flatPosition) {
				if (myRMO.VarRMO[0] < RMOShortEntryLevel) {
					downArrowA++;
				}
				if (myRMO.VarST3[0] > myRMO.VarST2[0]) {
					downArrowB++;
				}
				if (myRMO.VarST3[0] < -3) {
					downArrowC++;
				}
				
				if (myRMO.VarRMO[0] > RMOLongEntryLevel) {
					upArrowA++;
				}
				if (myRMO.VarST2[0] > myRMO.VarST3[0]) {
					upArrowB++;
				}
				if (myRMO.VarST3[0] > 3) {
					upArrowC++;
				}
			
				if ((downArrowA + downArrowB + downArrowC) == 3) {
					DrawArrowDown(CurrentBar.ToString(), false, 0, High[0] + (4 * TickSize), Color.IndianRed);
					shortEntry = true;
					flatPosition = false;
					if (soundAlert) {
						Alert(CurrentBar.ToString(), Priority.High, "Short Entry Alert", @EntryAlertFile, alertInterval, Color.Red, Color.White);
					}
				}
	
				if ((upArrowA + upArrowB + upArrowC) == 3) {
					DrawArrowUp(CurrentBar.ToString(), false, 0, Low[0] - (4 * TickSize), Color.RoyalBlue);
					longEntry = true;
					flatPosition = false;
					if (soundAlert) {
						Alert(CurrentBar.ToString(), Priority.High, "Long Entry Alert", @EntryAlertFile, alertInterval, Color.Blue, Color.White);
					}
				}
			}
			else if (longEntry) {
				if (CrossBelow(BollingerPercentB(Close, 20, 2).PctB, BPBLongExitLevel, 2)) {
					DrawDiamond(CurrentBar.ToString(), false, 0, High[0] + (5 * TickSize), Color.RoyalBlue);
					longEntry = false;
					flatPosition = true;
					if (soundAlert) {
						Alert(CurrentBar.ToString(), Priority.High, "Long Exit Alert", @ExitAlertFile, alertInterval, Color.Blue, Color.White);
					}
				}
			}
			else if (shortEntry) {
				if (CrossAbove(BollingerPercentB(Close, 20, 2).PctB, BPBShortExitLevel, 2)) {
					DrawDiamond(CurrentBar.ToString(), false, 0, Low[0] - (5 * TickSize), Color.IndianRed);
					shortEntry = false;
					flatPosition = true;
					if (soundAlert) {
						Alert(CurrentBar.ToString(), Priority.High, "Short Exit Alert", @ExitAlertFile, alertInterval, Color.Blue, Color.White);
					}
				}
			}
			
			if ((myRMO.VarRMO[0] > 0) && (myRMO.VarST2[0] > 0) && (myRMO.VarST3[0] > 0) &&
				(!myCZI.Sideways)) {
				if (colorBars) {
					BarColor = Color.Blue;
				}
			}
			else if ((myRMO.VarRMO[0] < 0) && (myRMO.VarST2[0] < 0) && (myRMO.VarST3[0] < 0) &&
				(!myCZI.Sideways)) {
				if (colorBars) {
					BarColor = Color.Red;
				}
			}
			else {
				if (colorBars) {
					BarColor = Color.Gray;
				}
			}
        }

        #region Properties
		[Description("BPB Long Exit Level")]
        [Category("Parameters")]
        public int BPBLongExitLevel
        {
            get { return bpbLongExitLevel; }
            set { bpbLongExitLevel = value; }
        }
		
		[Description("BPB Short Exit Level")]
        [Category("Parameters")]
        public int BPBShortExitLevel
        {
            get { return bpbShortExitLevel; }
            set { bpbShortExitLevel = value; }
        }
		
		[Description("RMO Long Entry Level")]
        [Category("Parameters")]
        public int RMOLongEntryLevel
        {
            get { return rmoLongEntryLevel; }
            set { rmoLongEntryLevel = value; }
        }
		
		[Description("RMO Short Entry Level")]
        [Category("Parameters")]
        public int RMOShortEntryLevel
        {
            get { return rmoShortEntryLevel; }
            set { rmoShortEntryLevel = value; }
        }
		
		[Description("")]
        [Category("Parameters")]
        public int Len1
        {
            get { return len1; }
            set { len1 = Math.Max(1, value); }
        }

        [Description("")]
        [Category("Parameters")]
        public int Len2
        {
            get { return len2; }
            set { len2 = Math.Max(1, value); }
        }

        [Description("")]
        [Category("Parameters")]
        public int Len3
        {
            get { return len3; }
            set { len3 = Math.Max(1, value); }
        }

        [Description("")]
        [Category("Parameters")]
        public int Len4
        {
            get { return len4; }
            set { len4 = Math.Max(1, value); }
        }
		
		[Description("How often do you want the alert to sound?")]
		[Category("Sound and Display")]
		[Gui.Design.DisplayName("Alert Interval (sec.)")]
		public int AlertInterval
		{
			get { return alertInterval; }
			set { alertInterval = value; }
		}
		
		[Description("Where is the ENTRY alert sound file located?  File name of .wav file to play. Provide either the absolute file path or just the name if the file is located in NinjaTrader Installation Folder sounds folder")]
		[Category("Sound and Display")]
		[Gui.Design.DisplayName("Alert .wav File")]
		public string EntryAlertFile
		{
			get { return entryAlertFile; }
			set { entryAlertFile = value; }
		}

		[Description("Where is the EXIT alert sound file located?  File name of .wav file to play. Provide either the absolute file path or just the name if the file is located in NinjaTrader Installation Folder sounds folder")]
		[Category("Sound and Display")]
		[Gui.Design.DisplayName("Alert .wav File")]
		public string ExitAlertFile
		{
			get { return exitAlertFile; }
			set { exitAlertFile = value; }
		}
		
		[Description("Sound an alert when an RMO Pattern is signaled")]
		[Category("Sound and Display")]
		[Gui.Design.DisplayName("Alert Active?")]
		public bool SoundAlert
		{
			get { return soundAlert; }
			set { soundAlert = value; }
		}
		
		[Description("Color price bars mirrored from histogram data.")]
		[Category("Sound and Display")]
		[Gui.Design.DisplayName("Color Price Bars?")]
		public bool ColorBars
		{
			get { return colorBars; }
			set { colorBars = value; }
		}
		#endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private RMOSystem[] cacheRMOSystem = null;

        private static RMOSystem checkRMOSystem = new RMOSystem();

        /// <summary>
        /// RMO System Indicator v3.0.0 - written by Randy Terbush, randy@terbush.org
        /// </summary>
        /// <returns></returns>
        public RMOSystem RMOSystem(int bPBLongExitLevel, int bPBShortExitLevel, int len1, int len2, int len3, int len4, int rMOLongEntryLevel, int rMOShortEntryLevel)
        {
            return RMOSystem(Input, bPBLongExitLevel, bPBShortExitLevel, len1, len2, len3, len4, rMOLongEntryLevel, rMOShortEntryLevel);
        }

        /// <summary>
        /// RMO System Indicator v3.0.0 - written by Randy Terbush, randy@terbush.org
        /// </summary>
        /// <returns></returns>
        public RMOSystem RMOSystem(Data.IDataSeries input, int bPBLongExitLevel, int bPBShortExitLevel, int len1, int len2, int len3, int len4, int rMOLongEntryLevel, int rMOShortEntryLevel)
        {
            checkRMOSystem.BPBLongExitLevel = bPBLongExitLevel;
            bPBLongExitLevel = checkRMOSystem.BPBLongExitLevel;
            checkRMOSystem.BPBShortExitLevel = bPBShortExitLevel;
            bPBShortExitLevel = checkRMOSystem.BPBShortExitLevel;
            checkRMOSystem.Len1 = len1;
            len1 = checkRMOSystem.Len1;
            checkRMOSystem.Len2 = len2;
            len2 = checkRMOSystem.Len2;
            checkRMOSystem.Len3 = len3;
            len3 = checkRMOSystem.Len3;
            checkRMOSystem.Len4 = len4;
            len4 = checkRMOSystem.Len4;
            checkRMOSystem.RMOLongEntryLevel = rMOLongEntryLevel;
            rMOLongEntryLevel = checkRMOSystem.RMOLongEntryLevel;
            checkRMOSystem.RMOShortEntryLevel = rMOShortEntryLevel;
            rMOShortEntryLevel = checkRMOSystem.RMOShortEntryLevel;

            if (cacheRMOSystem != null)
                for (int idx = 0; idx < cacheRMOSystem.Length; idx++)
                    if (cacheRMOSystem[idx].BPBLongExitLevel == bPBLongExitLevel && cacheRMOSystem[idx].BPBShortExitLevel == bPBShortExitLevel && cacheRMOSystem[idx].Len1 == len1 && cacheRMOSystem[idx].Len2 == len2 && cacheRMOSystem[idx].Len3 == len3 && cacheRMOSystem[idx].Len4 == len4 && cacheRMOSystem[idx].RMOLongEntryLevel == rMOLongEntryLevel && cacheRMOSystem[idx].RMOShortEntryLevel == rMOShortEntryLevel && cacheRMOSystem[idx].EqualsInput(input))
                        return cacheRMOSystem[idx];

            RMOSystem indicator = new RMOSystem();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.BPBLongExitLevel = bPBLongExitLevel;
            indicator.BPBShortExitLevel = bPBShortExitLevel;
            indicator.Len1 = len1;
            indicator.Len2 = len2;
            indicator.Len3 = len3;
            indicator.Len4 = len4;
            indicator.RMOLongEntryLevel = rMOLongEntryLevel;
            indicator.RMOShortEntryLevel = rMOShortEntryLevel;
            indicator.SetUp();

            RMOSystem[] tmp = new RMOSystem[cacheRMOSystem == null ? 1 : cacheRMOSystem.Length + 1];
            if (cacheRMOSystem != null)
                cacheRMOSystem.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheRMOSystem = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// RMO System Indicator v3.0.0 - written by Randy Terbush, randy@terbush.org
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.RMOSystem RMOSystem(int bPBLongExitLevel, int bPBShortExitLevel, int len1, int len2, int len3, int len4, int rMOLongEntryLevel, int rMOShortEntryLevel)
        {
            return _indicator.RMOSystem(Input, bPBLongExitLevel, bPBShortExitLevel, len1, len2, len3, len4, rMOLongEntryLevel, rMOShortEntryLevel);
        }

        /// <summary>
        /// RMO System Indicator v3.0.0 - written by Randy Terbush, randy@terbush.org
        /// </summary>
        /// <returns></returns>
        public Indicator.RMOSystem RMOSystem(Data.IDataSeries input, int bPBLongExitLevel, int bPBShortExitLevel, int len1, int len2, int len3, int len4, int rMOLongEntryLevel, int rMOShortEntryLevel)
        {
            return _indicator.RMOSystem(input, bPBLongExitLevel, bPBShortExitLevel, len1, len2, len3, len4, rMOLongEntryLevel, rMOShortEntryLevel);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// RMO System Indicator v3.0.0 - written by Randy Terbush, randy@terbush.org
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.RMOSystem RMOSystem(int bPBLongExitLevel, int bPBShortExitLevel, int len1, int len2, int len3, int len4, int rMOLongEntryLevel, int rMOShortEntryLevel)
        {
            return _indicator.RMOSystem(Input, bPBLongExitLevel, bPBShortExitLevel, len1, len2, len3, len4, rMOLongEntryLevel, rMOShortEntryLevel);
        }

        /// <summary>
        /// RMO System Indicator v3.0.0 - written by Randy Terbush, randy@terbush.org
        /// </summary>
        /// <returns></returns>
        public Indicator.RMOSystem RMOSystem(Data.IDataSeries input, int bPBLongExitLevel, int bPBShortExitLevel, int len1, int len2, int len3, int len4, int rMOLongEntryLevel, int rMOShortEntryLevel)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.RMOSystem(input, bPBLongExitLevel, bPBShortExitLevel, len1, len2, len3, len4, rMOLongEntryLevel, rMOShortEntryLevel);
        }

    }
}
#endregion
